/// <reference types="node" />
import { FinalizedFunc } from './RDD';
import '@dcfjs/common/noCaptureEnv';
import { FunctionEnv } from './../common/serializeFunction';
import { WorkDispatcher } from '@dcfjs/master/workerManager';
import { Context } from './Context';
import sr = require('@dcfjs/common/storageRegistry');
export declare type ParallelTask = (dispachWorker: WorkDispatcher) => any;
export declare type PartitionFunc<T> = (paritionId: number) => (workerId: string) => T | Promise<T>;
export declare type FinalizedFunc = (ts: sr.TempStorageSession) => void | Promise<void>;
export declare type MapperFunc<T, T1> = (input: T[]) => T1[];
export declare type ComparableType = string | number;
export declare type RDDFuncs<T> = [number, PartitionFunc<T>, FinalizedFunc | undefined];
export declare abstract class RDD<T> {
    protected _context: Context;
    protected constructor(context: Context);
    abstract getFunc(): RDDFuncs<T[]> | Promise<RDDFuncs<T[]>>;
    getNumPartitions(): number | Promise<number>;
    union(...others: RDD<T>[]): RDD<T>;
    collect(): Promise<T[]>;
    count(): Promise<number>;
    take(count: number): Promise<T[]>;
    max(this: RDD<number>, comparator?: (a: number, b: number) => number): Promise<T | undefined>;
    max(comparator: (a: T, b: T) => number): Promise<T | undefined>;
    min(this: RDD<number>, comparator?: (a: number, b: number) => number): Promise<T | undefined>;
    min(comparator: (a: T, b: T) => number): Promise<T | undefined>;
    mapPartitions<T1>(transformer: (input: T[]) => T1[], env?: FunctionEnv): RDD<T1>;
    glom(): RDD<T[]>;
    map<T1>(transformer: (input: T) => T1, env?: FunctionEnv): RDD<T1>;
    flatMap<T1>(transformer: (input: T) => T1[], env?: FunctionEnv): RDD<T1>;
    filter(filterFunc: (input: T) => boolean, env?: FunctionEnv): RDD<T>;
    saveAsTextFile(baseUrl: string, { overwrite, encoding, extension, compressor, functionEnv, }?: {
        overwrite?: boolean;
        encoding?: string;
        extension?: string;
        compressor?: (data: Buffer) => Buffer | Promise<Buffer>;
        functionEnv?: FunctionEnv;
    }): Promise<void>;
    reduce(reduceFunc: (a: T, b: T) => T, env?: FunctionEnv): Promise<T | undefined>;
    persist(storageType?: string): CachedRDD<T>;
    cache(storageType?: string): CachedRDD<T>;
    partitionBy(numPartitions: number, partitionFunc: (v: T) => number, env?: FunctionEnv): RDD<T>;
    coalesce(numPartitions: number): RDD<T>;
    repartition(numPartitions: number): RDD<T>;
    distinct(numPartitions?: number): RDD<T>;
    combineByKey<K, V, C>(this: RDD<[K, V]>, createCombiner: (a: V) => C, mergeValue: (a: C, b: V) => C, mergeCombiners: (a: C, b: C) => C, numPartitions?: number, partitionFunc?: (v: K) => number, env?: FunctionEnv): RDD<[K, C]>;
    reduceByKey<K, V>(this: RDD<[K, V]>, func: (a: V, B: V) => V, numPartitions?: number, partitionFunc?: (v: K) => number, env?: FunctionEnv): RDD<[K, V]>;
    groupWith<K, V, V1>(this: RDD<[K, V]>, other1: RDD<[K, V1]>): RDD<[K, [V[], V1[]]]>;
    groupWith<K, V, V1, V2>(this: RDD<[K, V]>, other1: RDD<[K, V1]>, other2: RDD<[K, V2]>): RDD<[K, [V[], V1[], V2[]]]>;
    groupWith<K, V, V1, V2, V3>(this: RDD<[K, V]>, other1: RDD<[K, V1]>, other2: RDD<[K, V2]>, other3: RDD<[K, V3]>): RDD<[K, [V[], V1[], V2[], V3[]]]>;
    groupWith<K>(this: RDD<[K, any]>, ...others: RDD<[K, any]>[]): RDD<[K, any]>;
    cogroup<K, V, V1>(this: RDD<[K, V]>, other: RDD<[K, V1]>, numPartitions?: number): RDD<[K, [V[], V1[]]]>;
    join<K, V, V1>(this: RDD<[K, V]>, other: RDD<[K, V1]>, numPartitions?: number): RDD<[K, [V, V1]]>;
    leftOuterJoin<K, V, V1>(this: RDD<[K, V]>, other: RDD<[K, V1]>, numPartitions?: number): RDD<[K, [V, V1 | null]]>;
    rightOuterJoin<K, V, V1>(this: RDD<[K, V]>, other: RDD<[K, V1]>, numPartitions?: number): RDD<[K, [V | null, V1]]>;
    fullOuterJoin<K, V, V1>(this: RDD<[K, V]>, other: RDD<[K, V1]>, numPartitions?: number): RDD<[K, [V | null, V1 | null]]>;
    sort<K extends ComparableType>(this: RDD<K>, ascending?: boolean, numPartitions?: number): RDD<K>;
    sortBy<K extends ComparableType>(keyFunc: (data: T) => K, ascending?: boolean, numPartitions?: number, env?: FunctionEnv): RDD<T>;
}
export declare class GeneratedRDD<T> extends RDD<T> {
    private _numPartition;
    private _function?;
    constructor(context: Context, numPartition: 0);
    constructor(context: Context, numPartition: number, func: (paritionId: number) => () => T[] | Promise<T[]>);
    getFunc(): RDDFuncs<T[]>;
    getNumPartitions(): number;
}
export declare class MappedRDD<T1, T> extends RDD<T1> {
    private _mapper;
    private _dependence;
    constructor(context: Context, dependence: RDD<T>, mapper: MapperFunc<T, T1>);
    getFunc(): Promise<RDDFuncs<T1[]>>;
    getNumPartitions(): number | Promise<number>;
}
export declare class UnionRDD<T> extends RDD<T> {
    private _dependences;
    constructor(context: Context, dependences: RDD<T>[]);
    getFunc(): Promise<RDDFuncs<T[]>>;
    getNumPartitions(): number | Promise<number>;
}
export declare class CachedRDD<T> extends RDD<T> {
    private _dependence;
    private _cachedNumPartition?;
    private _cachedPartitions?;
    private _storageType;
    private _autoUnpersist;
    constructor(context: Context, dependence: RDD<T>, storageType: string);
    autoUnpersist(): void;
    unpersist(): Promise<void>;
    getFunc(): Promise<RDDFuncs<T[]>>;
    getNumPartitions(): number | Promise<number>;
}
export declare class RepartitionRDD<T> extends RDD<T> {
    private _numPartition;
    private _dependence;
    private _partitionFunc;
    constructor(context: Context, dependence: RDD<T>, numPartition: number, partitionFunc: (paritionId: number) => (v: T[]) => T[][]);
    getFunc(): Promise<RDDFuncs<T[]>>;
    getNumPartitions(): number;
}
export declare class SortedRDD<T, K extends ComparableType> extends RDD<T> {
    private _numPartition;
    private _dependence;
    private _keyFunc;
    private _ascending;
    constructor(context: Context, dependence: RDD<T>, numPartition: number, keyFunc: (data: T) => K, ascending: boolean);
    getFunc(): Promise<RDDFuncs<T[]>>;
    getNumPartitions(): number;
}
export declare class CoalesceRDD<T> extends RDD<T> {
    private _numPartition;
    private _dependence;
    constructor(context: Context, dependence: RDD<T>, numPartition: number);
    getFunc(): Promise<RDDFuncs<T[]>>;
    getNumPartitions(): number;
}
export declare class FileLoaderRDD extends RDD<[string, Buffer]> {
    private _loaderFunc;
    private _fileListFunc;
    private _fileList?;
    constructor(context: Context, fileListFunc: () => string[] | Promise<string[]>, loaderFunc: (fileName: string) => Buffer | Promise<Buffer>);
    private _getFileList;
    getFunc(): Promise<RDDFuncs<[string, Buffer][]>>;
    getNumPartitions(): Promise<number>;
}
